<?php

include "config.php";
include "helper.php";

$user = is_logged_in();

if ($user === false) {
  include "boilerplate/head.php";
  echo "<a href=\"login.php\">{$lang['account']['log_in']}</a>";

  $q = "SELECT count(*) FROM users LIMIT 1;";

  if ($signups !== false || ($signups === false && pg_fetch_array(pg_query($db, $q))["count"] !== 0)) {
    echo " - <a href=\"signup.php\">{$lang['account']['sign_up']}</a>";
  }

  include "boilerplate/foot.php";
  exit();
} else if ($_SERVER["REQUEST_METHOD"] === "POST") {
  $intent = $_POST["intent"];

  if ($intent === "create-item") {
    $row_id = $_POST["row"];
    $date = strtotime($_POST["date"]);
    $description = $_POST["description"];

    if (strlen($row_id) !== 64 || !preg_match("/[a-f0-9]{64}/", $row_id)) {
      $err = lang["add"]["errors"]["invalid_id"];
    } else if ($date === false) {
      $err = lang["add"]["errors"]["date"];
    } else if (strlen($description) === 0 || strlen($description) > 256) {
      $err = lang["add"]["errors"]["description_length"];
    } else {
      $repopulate = array(
        "row_id" => $row_id,
        "date" => $date
      );

      pg_insert(
        $db, "item", array(
          "id" => generate_id(),
          "row" => $row_id,
          "description" => $description,
          "date" => date("Y-m-d", $date)
        )
      );
    }
  } else if ($intent === "change-password") {
    $old_pw = $_POST["old"];
    $new_pw = $_POST["new"];
    $confirm_pw = $_POST["confirm"];

    if (!($old_pw && $new_pw && $confirm_pw)) {
      $err = $lang["account"]["errors"]["bad_request"];
    } else if ($new_pw !== $confirm_pw) {
      $err = $lang["account"]["errors"]["password_match"];
    } else if (password_verify($old_pw, $user["password_hash"])) {
      $pw_hash = password_hash($new_pw, PASSWORD_DEFAULT);

      pg_update(
        $db, "users",
        array("password_hash" => $pw_hash),
        array("id" => $user["id"])
      );

      $token = get_token($user["username"], $pw_hash);
      setcookie(
        "token", $token,
        time() + 60 * 60 * 24 * 365
      );

      $conf = $lang["account"]["errors"]["password_changed"];
    } else {
      $err = $lang["account"]["errors"]["incorrect_password"];
    }
  }
} else if ($_GET["del"] && strlen($_GET["del"]) === 64 && preg_match("/[a-f0-9]{64}/", $_GET["del"])) {
  pg_query($db, "DELETE FROM item WHERE id='{$_GET['del']}';");
}

include "boilerplate/head.php";

$select_options = "";
$todo_list = "";

$q = "SELECT * FROM section WHERE users='{$user['id']}' ORDER BY LOWER(name) ASC;";
$sections = pg_fetch_all(pg_query($db, $q));

foreach ($sections as $section) {
  $q = "SELECT * FROM row WHERE section='{$section['id']}' ORDER BY LOWER(name) ASC;";
  $rows = pg_fetch_all(pg_query($db, $q));
  $show_subtitle = $section["show_subtitle"] === "t";
  $enable_optgroup = $show_subtitle || count($rows) !== 1;

  $todo_list .= "<h3>" . htmlspecialchars($section["name"]) . "</h3>";

  if ($enable_optgroup) {
    $select_options .= "<optgroup label=\"" . htmlspecialchars($section["name"]) . "\">";
  }

  $first = true;
  foreach ($rows as $row) {
    if ($repopulate["row_id"] == $row["id"]) {
      $select_options .= "<option value=\"{$row['id']}\" selected>";
    } else {
      $select_options .= "<option value=\"{$row['id']}\">";
    }

    if ($first && !$show_subtitle) {
      $select_options .= htmlspecialchars($section["name"]);
    } else {
      $select_options .= htmlspecialchars($row["name"]);
      $todo_list .= "<strong>" . htmlspecialchars($row["name"]) . "</strong>";
    }

    $q = "SELECT * FROM item WHERE row='{$row['id']}' ORDER BY date ASC, LOWER(description) ASC;";
    $items = pg_fetch_all(pg_query($db, $q));
    $todo_list .= "<ul>";

    if (count($items) === 0) {
      $todo_list .= "<li><i>{$lang['list']['none']}</i></li>";
    } else {
      foreach ($items as $item) {
        $color = "red";

        $todo_list .= "<li>" . str_replace(
          "%d", htmlspecialchars($item["description"]), str_replace(
            "%c", "</span>", str_replace(
              "%C", "<span class=\"$color\">", str_replace(
                "%t", date("d M, Y", strtotime($item["date"])), $row["display_format"]
              )
            )
          )
        ) . " <a class=\"plain\" href=\"index.php?del={$item['id']}\"><button tabindex=\"-1\">{$lang['list']['remove']}</button></a></li>";
      }
    }

    $todo_list .= "</ul>";
    $select_options .= "</option>";
    $first = false;
  }

  if ($enable_optgroup) {
    $select_options .= "</optgroup>";
  }
}

?>

<div id="container">
  <h2><?php echo $lang["add"]["title"]; ?></h2>
  <form method="POST">
    <input type="hidden" name="intent" value="create-item">
    <table>
      <tr>
        <td class="right"><label for="row"><?php echo $lang["add"]["section"]; ?></label></td>
        <td>
          <select required id="row" name="row">
            <?php echo $select_options; ?>
          </select>
        </td>
      </tr>
      <tr>
        <td class="right"><label for="date"><?php echo $lang["add"]["date"]; ?></label></td>
        <td><input <?php echo get_pop("date", "date"); ?> type="date" name="date" id="date" required></td>
      </tr>
      <tr>
        <td class="right"><label for="description"><?php echo $lang["add"]["description"]; ?></label></td>
        <td><input autofocus maxlength="256" placeholder="<?php echo $lang["add"]["description_placeholder"]; ?>" name="description" id="description" required></td>
      </tr>
      <tr>
        <td></td>
        <td><input type="submit" value="<?php echo $lang["add"]["button"]; ?>"></td>
      </tr>
    </table>
  </form>

  <h2><?php echo $lang["list"]["title"]; ?></h2>
  <?php echo $todo_list; ?>

  <details open>
    <summary><?php echo $lang["settings"]["title"]; ?></summary>

    <div id="settings-container">
      <div>
        <h3><?php echo $lang["settings"]["sections"]["title"]; ?></h3>
      </div>

      <div>
        <h3><?php echo $lang["settings"]["account"]["title"]; ?></h3>
        <?php echo str_replace("%u", "<code>{$user['username']}</code>", $lang["settings"]["account"]["current"]); ?>
        <a href="logout.php"><?php echo $lang["settings"]["account"]["log_out"]; ?></a>

        <h4><?php echo $lang["settings"]["account"]["password"]["title"]; ?></h4>
        <form method="POST">
          <input type="hidden" name="intent" value="change-password">
          <table>
            <tr>
              <td class="right"><label for="old-pw"><?php echo $lang["settings"]["account"]["password"]["current"]; ?></td>
              <td><input type="password" id="old-pw" name="old" placeholder="<?php echo $lang["settings"]["account"]["password"]["old_placeholder"]; ?>"></td>
            </tr>
            <tr>
              <td class="right"><label for="new-pw"><?php echo $lang["settings"]["account"]["password"]["new"]; ?></td>
              <td><input type="password" id="new-pw" name="new" placeholder="<?php echo $lang["settings"]["account"]["password"]["new_placeholder"]; ?>"></td>
            </tr>
            <tr>
              <td class="right"><label for="confirm-pw"><?php echo $lang["settings"]["account"]["password"]["confirm"]; ?></td>
              <td><input type="password" id="confirm-pw" name="confirm" placeholder="<?php echo $lang["settings"]["account"]["password"]["new_placeholder"]; ?>"></td>
            </tr>
            <tr>
              <td></td>
              <td><input type="submit" value="<?php echo $lang["settings"]["account"]["password"]["button"]; ?>"></td>
            </tr>
          </table>
        </form>
      </div>
    </div>
    <p><a href="https://git.trinkey.com/trinkey/tlm"><?php echo $lang["settings"]["source_code"]; ?></a><p>
  </details>
</div>

<?php include "boilerplate/foot.php"; ?>
